from yougotmail.retrieve.retrieve_emails import RetrieveEmails
from yougotmail.ai._ai_prompts import AGENT_INSTRUCTIONS
from yougotmail.ai._ai_tools import AI_TOOLS, TOOLS
from typing import Any, Dict


class AI:
    def __init__(self, client_id, client_secret, tenant_id, open_ai_api_key=None):
        self.retrieve_emails = RetrieveEmails(client_id, client_secret, tenant_id)
        self.open_ai_api_key = open_ai_api_key
        self.client_id = client_id
        self.client_secret = client_secret
        self.tenant_id = tenant_id

    def _ensure_ai_handler(self):
        """Lazy import of AI Handler to avoid immediate OpenAI dependency"""
        if not self.open_ai_api_key:
            raise ValueError(
                "OpenAI API key is required for AI functionality. Please provide it when initializing the YouGotMail class."
            )

        try:
            from yougotmail.ai._ai_handler import AIHandler

            return AIHandler(
                open_ai_api_key=self.open_ai_api_key
            )
        except ImportError:
            raise ImportError(
                "OpenAI package is not installed. Install it with 'pip install yougotmail[openai]'"
            )

    def ai_structured_output_from_email_body(
        self, *, email_body: str, schema: Dict[str, Any]
    ):
        ai_handler = self._ensure_ai_handler()
        schema = {
            "type": "json_schema",
            "json_schema": {
                "name": "email_schema",
                "description": "A schema for answering questions about the email and its contents",
                "schema": {
                    "type": "object",
                    "properties": {
                        **schema,
                    },
                    "required": list(schema.keys()),
                    "additionalProperties": False,
                },
            },
        }

        content_for_ai = f"""
            Here is the email content: {email_body}
            """

        return ai_handler.structured_outputs(
            prompt=f"Extract the following information from the email: {schema}",
            schema=schema,
            content=content_for_ai,
            model="gpt-4.1",
        )

    def ai_get_emails_with_structured_output(
        self,
        *,
        inbox=[],
        range="",
        start_date="",  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        start_time="",
        end_date="",  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        end_time="",
        subject=[],
        sender_name=[],
        sender_address=[],
        recipients=[],
        cc=[],
        bcc=[],
        folder_path=[],
        drafts=False,
        archived=False,
        deleted=False,
        sent=False,
        read="all",
        attachments=True,
        storage=None,
        schema={},
    ):
        inboxes = self.retrieve_emails.get_emails(
            inbox=inbox,
            range=range,
            start_date=start_date,
            start_time=start_time,
            end_date=end_date,
            end_time=end_time,
            subject=subject,
            sender_name=sender_name,
            sender_address=sender_address,
            recipients=recipients,
            cc=cc,
            bcc=bcc,
            folder_path=folder_path,
            drafts=drafts,
            archived=archived,
            deleted=deleted,
            sent=sent,
            read=read,
            attachments=attachments,
            storage=storage,
        )
        inbox_list = []
        for inbox in inboxes:
            emails = inbox.get("emails")
            emails_list = []
            for email in emails:
                email_body = email.get("body")
                email["structured_output"] = self.ai_structured_output_from_email_body(
                    email_body=email_body, schema=schema
                )
                emails_list.append(email)
            inbox["emails"] = emails_list
            inbox_list.append(inbox)
        return inbox_list

    def ai_agent_with_tools(self, inbox, prompt):
        try:
            ai_handler = self._ensure_ai_handler()
            return ai_handler.function_calling(
                instructions=AGENT_INSTRUCTIONS,
                prompt=prompt,
                model="gpt-4.1",
                tools=TOOLS,
                ai_tools=AI_TOOLS(self.client_id, self.client_secret, self.tenant_id, inbox),
            )
        except Exception as e:
            print(f"Error in ai_agent_with_tools: {e}")
            return None



from yougotmail.quick.quick import Quick
from yougotmail.retrieve.retrieve_emails import RetrieveEmails
from yougotmail.retrieve.retrieve_conversations import RetrieveConversations
from yougotmail.retrieve.retrieve_attachments import RetrieveAttachments
from yougotmail.send.send import Send
from yougotmail.send.reply import Reply
from yougotmail._utils._ms_webhook import MSWebhook
from yougotmail._utils._validation import (
    validate_inputs,
    EMAIL_VALIDATION_RULES,
    ValidationError,
)

class YouGotMail:
    def __init__(
        self,
        client_id,
        client_secret,
        tenant_id,
        open_ai_api_key="",
        mongo_url="",
        mongo_db_name="",
        email_collection="",
        conversation_collection="",
        attachment_collection="",
        aws_access_key_id="",
        aws_secret_access_key="",
        region_name="",
        bucket_name="",
    ):
        if client_id is None or client_secret is None or tenant_id is None:
            raise ValueError("client_id, client_secret and tenant_id are required")

        self.send = Send(client_id, client_secret, tenant_id)
        self.retrieve_emails = RetrieveEmails(
            client_id,
            client_secret,
            tenant_id,
            mongo_url=mongo_url,
            mongo_db_name=mongo_db_name,
            email_collection=email_collection,
            conversation_collection=conversation_collection,
            attachment_collection=attachment_collection,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name,
            bucket_name=bucket_name,
        )
        self.retrieve_conversations = RetrieveConversations(
            client_id,
            client_secret,
            tenant_id,
            mongo_url=mongo_url,
            mongo_db_name=mongo_db_name,
            email_collection=email_collection,
            conversation_collection=conversation_collection,
            attachment_collection=attachment_collection,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name,
            bucket_name=bucket_name,
        )
        self.retrieve_attachments = RetrieveAttachments(
            client_id,
            client_secret,
            tenant_id,
            mongo_url=mongo_url,
            mongo_db_name=mongo_db_name,
            email_collection=email_collection,
            conversation_collection=conversation_collection,
            attachment_collection=attachment_collection,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name,
            bucket_name=bucket_name,
        )
        self.quick = Quick(
            client_id,
            client_secret,
            tenant_id,
            mongo_url=mongo_url,
            mongo_db_name=mongo_db_name,
            email_collection=email_collection,
            conversation_collection=conversation_collection,
            attachment_collection=attachment_collection,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name,
            bucket_name=bucket_name,
        )
        self.reply = Reply(client_id, client_secret, tenant_id)
        # Store credentials for lazy AI initialization
        self._ai_credentials = {
            "client_id": client_id,
            "client_secret": client_secret,
            "tenant_id": tenant_id,
            "open_ai_api_key": open_ai_api_key,
        }
        self._ai_instance = None
        self.ms_webhook = MSWebhook(client_id, client_secret, tenant_id)

    def _get_ai(self):
        """Lazy initialization of AI component"""
        if self._ai_instance is None:
            try:
                from yougotmail.ai.ai import AI

                self._ai_instance = AI(**self._ai_credentials)
            except ImportError:
                raise ImportError(
                    "OpenAI dependencies are not installed. Install them with 'pip install yougotmail[openai]'"
                )
        return self._ai_instance

    # Functions handling email retrieval and conversation retrieval
    @validate_inputs(**EMAIL_VALIDATION_RULES)
    def get_emails(
        self,
        *,
        inbox=[],
        range="",
        start_date="",  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        start_time="",
        end_date="",  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        end_time="",
        subject=[],
        sender_name=[],
        sender_address=[],
        recipients=[],
        cc=[],
        bcc=[],
        folder_path="",
        drafts=False,
        archived=False,
        deleted=False,
        sent=False,
        read="all",
        attachments=False,
        storage=None
    ):
        # Add custom validation logic specific to this function
        if range and (start_date or end_date or start_time or end_time):
            raise ValidationError(
                "Cannot specify both 'range' and custom date/time parameters"
            )

        return self.retrieve_emails.get_emails(
            inbox=inbox,
            range=range,
            start_date=start_date,
            start_time=start_time,
            end_date=end_date,
            end_time=end_time,
            subject=subject,
            sender_name=sender_name,
            sender_address=sender_address,
            recipients=recipients,
            cc=cc,
            bcc=bcc,
            folder_path=folder_path,
            drafts=drafts,
            archived=archived,
            deleted=deleted,
            sent=sent,
            read=read,
            attachments=attachments,
            storage=storage,
        )

    def get_conversation(
        self,
        inbox="",
        conversation_id="",
        range="last_365_days",
        start_date="",
        start_time="",
        end_date="",
        end_time="",
        subject="",
        sender_name="",
        sender_address="",
        read="all",
        attachments=False,
        storage=None
    ):
        return self.retrieve_conversations.get_conversation(
            inbox=inbox,
            conversation_id=conversation_id,
            range=range,
            start_date=start_date,
            start_time=start_time,
            end_date=end_date,
            end_time=end_time,
            subject=subject,
            sender_name=sender_name,
            sender_address=sender_address,
            read=read,
            attachments=attachments,
            storage=storage,
        )

    def get_attachments(
        self,
        inbox=[],
        range="",
        start_date="",  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        start_time="",
        end_date="",  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        end_time="",
        subject=[],
        sender_name=[],
        sender_address=[],
        recipients=[],
        cc=[],
        bcc=[],
        folder_path=[],
        drafts=False,
        archived=False,
        deleted=False,
        sent=False,
        read="all",
        storage=None,
    ):
        return self.retrieve_attachments.get_attachments(
            inbox=inbox,
            range=range,
            start_date=start_date,
            start_time=start_time,
            end_date=end_date,
            end_time=end_time,
            subject=subject,
            sender_name=sender_name,
            sender_address=sender_address,
            recipients=recipients,
            cc=cc,
            bcc=bcc,
            folder_path=folder_path,
            drafts=drafts,
            archived=archived,
            deleted=deleted,
            sent=sent,
            read=read,
            storage=storage,
        )

    # Functions handling email sending
    def draft_email(
        self,
        inbox="",
        subject="",
        importance="",
        email_body="",
        to_recipients=[],
        cc_recipients=[],
        bcc_recipients=[],
        attachments=[],
    ):

        return self.send.draft_email(
            inbox=inbox,
            subject=subject,
            importance=importance,
            email_body=email_body,
            to_recipients=to_recipients,
            cc_recipients=cc_recipients,
            bcc_recipients=bcc_recipients,
            attachments=attachments,
        )

    def send_email_draft(self, inbox, id):
        return self.send.send_email_draft(inbox, id)

    def send_email(
        self,
        inbox="",
        subject="",
        importance="",
        email_body="",
        to_recipients=[],
        cc_recipients=[],
        bcc_recipients=[],
        attachments=[],
    ):
        return self.send.send_email(
            inbox,
            subject,
            importance,
            email_body,
            to_recipients,
            cc_recipients,
            bcc_recipients,
            attachments,
        )

    # Quick action functions for simplified usage
    def get_last_hour_emails(self, inbox):
        return self.quick.get_last_hour_emails(inbox)

    def get_unread_emails(self, inbox):
        return self.quick.get_unread_emails(inbox)

    def draft_reply_to_email(self, inbox="", email="", email_id=""):
        return self.reply.draft_reply_to_email(inbox, email, email_id)

    # Functions handling email replies
    def reply_to_email(
        self, inbox="", email_id="", email_body="", cc_recipients=[], bcc_recipients=[]
    ):
        return self.reply.reply_to_email(
            inbox, email_id, email_body, cc_recipients, bcc_recipients
        )

    def ai_structured_output_from_email_body(self, email_body, schema):
        ai = self._get_ai()
        return ai.ai_structured_output_from_email_body(
            email_body=email_body, schema=schema
        )

    @validate_inputs(**EMAIL_VALIDATION_RULES)
    def ai_get_emails_with_structured_output(
        self,
        *,
        inbox=[],
        range="",
        start_date="",  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        start_time="",
        end_date="",  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        end_time="",
        subject=[],
        sender_name=[],
        sender_address=[],
        recipients=[],
        cc=[],
        bcc=[],
        folder_path=[],
        drafts=False,
        archived=False,
        deleted=False,
        sent=False,
        read="all",
        attachments=True,
        storage=None,
        schema={}
    ):
        ai = self._get_ai()
        # Add custom validation logic specific to this function
        if range and (start_date or end_date or start_time or end_time):
            raise ValidationError(
                "Cannot specify both 'range' and custom date/time parameters"
            )

        return ai.ai_get_emails_with_structured_output(
            inbox=inbox,
            range=range,
            start_date=start_date,
            start_time=start_time,
            end_date=end_date,
            end_time=end_time,
            subject=subject,
            sender_name=sender_name,
            sender_address=sender_address,
            recipients=recipients,
            cc=cc,
            bcc=bcc,
            folder_path=folder_path,
            drafts=drafts,
            archived=archived,
            deleted=deleted,
            sent=sent,
            read=read,
            attachments=attachments,
            storage=storage,
            schema=schema,
        )

    def ai_agent_with_tools(self, inbox,prompt):
        ai = self._get_ai()
        return ai.ai_agent_with_tools(inbox=inbox, prompt=prompt)

    def create_microsoft_graph_webhook(self, inbox, api_url, client_state):
        return self.ms_webhook.create_microsoft_graph_webhook(
            inbox, api_url, client_state
        )

    def get_active_subscriptions_for_inbox(self, inbox):
        return self.ms_webhook.get_active_subscriptions_for_inbox(inbox)

    def delete_subscription(self, subscription_id):
        return self.ms_webhook.delete_subscription(subscription_id)

    def renew_subscriptions(self, inbox):
        return self.ms_webhook.renew_subscriptions(inbox)



import requests
import datetime
from dateutil.parser import parse as parse_date
from yougotmail._utils._utils import Utils
from yougotmail.retrieve.retrieve_attachments import RetrieveAttachments
from yougotmail.retrieve.retrieval_utils import RetrievalUtils


class RetrieveEmails:
    def __init__(
        self,
        client_id,
        client_secret,
        tenant_id,
        mongo_url="",
        mongo_db_name="",
        email_collection="",
        conversation_collection="",
        attachment_collection="",
        aws_access_key_id="",
        aws_secret_access_key="",
        region_name="",
        bucket_name="",
    ):
        self.utils = Utils()
        self.token = self.utils._generate_MS_graph_token(
            client_id, client_secret, tenant_id
        )

        # Store storage configuration but don't initialize yet
        self._storage_config = {
            "mongo_url": mongo_url,
            "mongo_db_name": mongo_db_name,
            "email_collection": email_collection,
            "conversation_collection": conversation_collection,
            "attachment_collection": attachment_collection,
            "aws_access_key_id": aws_access_key_id,
            "aws_secret_access_key": aws_secret_access_key,
            "region_name": region_name,
            "bucket_name": bucket_name,
        }
        self._db_storage = None

        self.retrieve_attachments = RetrieveAttachments(
            client_id, client_secret, tenant_id, **self._storage_config
        )
        self.retrieval_utils = RetrievalUtils(client_id, client_secret, tenant_id)

    def _ensure_storage(self):
        """Lazy initialization of storage"""
        if self._db_storage is None:
            from yougotmail.storage.storage import Storage

            self._db_storage = Storage(**self._storage_config)
        return self._db_storage

    @property
    def db_storage(self):
        """Property to access storage only when needed"""
        return self._ensure_storage()

    def get_emails(
        self,
        *,
        inbox,
        range,
        start_date,  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        start_time,
        end_date,  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        end_time,
        subject,
        sender_name,
        sender_address,
        recipients,
        cc,
        bcc,
        folder_path,
        drafts,
        archived,
        deleted,
        sent,
        read,
        attachments,
        storage,
    ):
        try:
            inboxes_list = []

            print(
                self.utils._display_query_details(
                    inbox,
                    range,
                    start_date,
                    start_time,
                    end_date,
                    end_time,
                    subject,
                    sender_name,
                    sender_address,
                    recipients,
                    cc,
                    bcc,
                    folder_path,
                    drafts,
                    archived,
                    deleted,
                    sent,
                    read,
                    attachments,
                    storage,
                )
            )

            for inbox in inbox:
                print(f"\nSearching for emails in inbox {inbox} ")
                emails_from_one_inbox = self._get_emails_from_one_inbox(
                    inbox,
                    range,
                    start_date,
                    start_time,
                    end_date,
                    end_time,
                    subject,
                    sender_name,
                    sender_address,
                    recipients,
                    cc,
                    bcc,
                    folder_path,
                    drafts,
                    archived,
                    deleted,
                    sent,
                    read,
                    attachments,
                )
                if emails_from_one_inbox is not None:
                    inboxes_list.append(emails_from_one_inbox)
                else:
                    return None

            if storage in ["emails", "emails_and_attachments"]:
                storage_instance = self._ensure_storage()
                if storage == "emails":
                    storage_instance.store_emails(inboxes_list)
                else:
                    storage_instance.store_emails_and_attachments(inboxes_list)
                inboxes_list = self.utils._convert_datetimes(
                    self.utils._remove_objectid_from_list(inboxes_list)
                )

            return inboxes_list

        except Exception as e:
            print(f"Error in get_emails: {e}")
            return None

    def _get_emails_from_one_inbox(
        self,
        inbox,
        range,
        start_date,
        start_time,
        end_date,
        end_time,
        subject,
        sender_name,
        sender_address,
        recipients,
        cc,
        bcc,
        folder_path,
        drafts,
        archived,
        deleted,
        sent,
        read,
        attachments,
    ):
        try:
            url_filter = self.retrieval_utils._create_filter_url(
                inbox=inbox,
                range=range,
                start_date=start_date,
                start_time=start_time,
                end_date=end_date,
                end_time=end_time,
                subject=subject,
                sender_name=sender_name,
                sender_address=sender_address,
                recipients=recipients,
                cc=cc,
                bcc=bcc,
                folder_path=folder_path,
                drafts=drafts,
                read=read,
            )

            url = (
                f"https://graph.microsoft.com/v1.0/users/{inbox}/messages?{url_filter}"
            )

            headers = {
                "Authorization": f"Bearer {self.token}",
                "Accept": "application/json",
                "Prefer": 'outlook.body-content-type="text"',
            }

            try:
                emails_list = []
                page_count = 0
                email_count = 0
                while url and page_count < 100000:
                    page_count += 1
                    response = requests.get(url, headers=headers)
                    response.raise_for_status()
                    data = response.json()
                    emails = data["value"]
                    next_page_link = data.get("@odata.nextLink")

                    for email in emails:
                        if not attachments:
                            attachment_list = []
                        elif attachments:
                            attachment_list = self.retrieve_attachments._get_attachments_from_one_email(
                                inbox, email
                            )
                        re_structured_email = self.retrieval_utils._re_structure_email(
                            email, inbox, attachment_list
                        )
                        filtered_email = self.retrieval_utils._filter_email_outputs(
                            re_structured_email, archived, deleted, sent
                        )
                        if filtered_email is not None:
                            email_count += 1
                            print(f"\nEmail {email_count} in {inbox}:")
                            print(f"  Inbox:     {inbox}")
                            print(
                                f"  From:      {filtered_email.get('sender_address')}"
                            )
                            print(f"  Subject:   {filtered_email.get('subject')}")
                            print(f"  Date:      {filtered_email.get('received_date')}")
                            print(f"  Folder:    {filtered_email.get('folder_name')}")
                            print(
                                f"  Attachments: {len(filtered_email.get('attachments', []))}"
                            )
                            emails_list.append(filtered_email)
                    url = next_page_link

            except requests.exceptions.RequestException as e:
                print(f"Error making request: {e}")
                return None

            if len(emails_list) == 0:
                result = {"inbox": inbox, "number_of_emails_found": 0, "emails": []}
                print(f"No emails found in inbox {inbox} for the given filters")
                return result

            # Sort emails by received_date, most recent first
            def safe_parse_date(email):
                try:
                    received_date = email.get("received_date")
                    if received_date:
                        return parse_date(received_date)
                    else:
                        return (
                            datetime.datetime.min
                        )  # Put emails with no date at the end
                except Exception:
                    return datetime.datetime.min  # Put unparseable dates at the end

            emails_list.sort(key=safe_parse_date, reverse=True)

            return {
                "inbox": inbox,
                "number_of_emails_found": len(emails_list),
                "emails": emails_list,
            }
        except Exception as e:
            print(f"Error in _get_emails_from_one_inbox: {e}")
            return None

    def _get_emails_from_one_inbox_without_print(
        self,
        inbox,
        range,
        start_date,
        start_time,
        end_date,
        end_time,
        subject,
        sender_name,
        sender_address,
        recipients,
        cc,
        bcc,
        folder_path,
        drafts,
        archived,
        deleted,
        sent,
        read,
        attachments,
    ):
        try:
            url_filter = self.retrieval_utils._create_filter_url(
                inbox=inbox,
                range=range,
                start_date=start_date,
                start_time=start_time,
                end_date=end_date,
                end_time=end_time,
                subject=subject,
                sender_name=sender_name,
                sender_address=sender_address,
                recipients=recipients,
                cc=cc,
                bcc=bcc,
                folder_path=folder_path,
                drafts=drafts,
                read=read,
            )

            url = (
                f"https://graph.microsoft.com/v1.0/users/{inbox}/messages?{url_filter}"
            )

            headers = {
                "Authorization": f"Bearer {self.token}",
                "Accept": "application/json",
                "Prefer": 'outlook.body-content-type="text"',
            }

            try:
                emails_list = []
                page_count = 0
                email_count = 0
                while url and page_count < 100000:
                    page_count += 1
                    response = requests.get(url, headers=headers)
                    response.raise_for_status()
                    data = response.json()
                    emails = data["value"]
                    next_page_link = data.get("@odata.nextLink")

                    for email in emails:
                        if not attachments:
                            attachment_list = []
                        elif attachments:
                            attachment_list = self.retrieve_attachments._get_attachments_from_one_email(
                                inbox, email
                            )
                        re_structured_email = self.retrieval_utils._re_structure_email(
                            email, inbox, attachment_list
                        )
                        filtered_email = self.retrieval_utils._filter_email_outputs(
                            re_structured_email, archived, deleted, sent
                        )
                        if filtered_email is not None:
                            emails_list.append(filtered_email)
                    url = next_page_link

            except requests.exceptions.RequestException as e:
                print(f"Error making request: {e}")
                return None

            if len(emails_list) == 0:
                result = {"inbox": inbox, "number_of_emails_found": 0, "emails": []}
                return result

            # Sort emails by received_date, most recent first
            def safe_parse_date(email):
                try:
                    received_date = email.get("received_date")
                    if received_date:
                        return parse_date(received_date)
                    else:
                        return (
                            datetime.datetime.min
                        )  # Put emails with no date at the end
                except Exception:
                    return datetime.datetime.min  # Put unparseable dates at the end

            emails_list.sort(key=safe_parse_date, reverse=True)

            return {
                "inbox": inbox,
                "number_of_emails_found": len(emails_list),
                "emails": emails_list,
            }
        except Exception as e:
            print(f"Error in _get_emails_from_one_inbox: {e}")
            return None



import requests
import base64
from yougotmail._utils._utils import Utils
import os
import json
from urllib.parse import urlparse, unquote


class Send:
    def __init__(self, client_id, client_secret, tenant_id):
        self.utils = Utils()
        self.token = self.utils._generate_MS_graph_token(
            client_id, client_secret, tenant_id
        )

    def draft_email(
        self,
        inbox="",
        subject="",
        importance="",
        email_body="",
        to_recipients=[],
        cc_recipients=[],
        bcc_recipients=[],
        attachments=[],
    ):

        url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages"

        headers = {
            "Authorization": f"Bearer {self.token}",
            "Content-type": "application/json",
        }

        if inbox == "":
            raise Exception("Inbox is required")

        # Fix: Don't overwrite the parameter arrays
        to_recipients_formatted = []
        for to_recipient in to_recipients:
            to_recipients_formatted.append({"emailAddress": {"address": to_recipient}})
        cc_recipients_formatted = []
        for cc_recipient in cc_recipients:
            cc_recipients_formatted.append({"emailAddress": {"address": cc_recipient}})
        bcc_recipients_formatted = []
        for bcc_recipient in bcc_recipients:
            bcc_recipients_formatted.append(
                {"emailAddress": {"address": bcc_recipient}}
            )

        data = {
            "subject": subject,
            "body": {"contentType": "HTML", "content": email_body},
            "toRecipients": to_recipients_formatted,
            "ccRecipients": cc_recipients_formatted,
            "bccRecipients": bcc_recipients_formatted,
        }
        if importance != "":
            data["importance"] = importance

        response = requests.post(url, headers=headers, json=data)
        response.raise_for_status()

        if attachments != []:
            self._add_attachments_to_email(inbox, response.json()["id"], attachments)

        return {
            "status": "success",
            "message": "Email drafted successfully",
            "id": response.json()["id"],
            "recipients": {
                "to": to_recipients,
                "cc": cc_recipients,
                "bcc": bcc_recipients,
            },
            "subject": subject,
        }

    def send_email_draft(self, inbox, id):
        if inbox == "":
            raise Exception("Inbox is required")
        url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{id}/send"

        headers = {
            "Authorization": f"Bearer {self.token}",
            "Content-type": "application/json",
        }

        try:
            response = requests.post(url, headers=headers)

            if response.status_code == 202:
                return {"status": "success", "message": "Email sent successfully"}
            else:
                # Handle error cases gracefully
                error_message = "Failed to send email"
                try:
                    error_data = response.json()
                    if "error" in error_data:
                        error_message = error_data["error"].get(
                            "message", error_message
                        )
                except (json.JSONDecodeError, ValueError, KeyError):
                    error_message = f"HTTP {response.status_code}: {response.reason}"

                return {
                    "status": "error",
                    "message": error_message,
                    "status_code": response.status_code,
                }
        except requests.exceptions.RequestException as e:
            return {"status": "error", "message": f"Request failed: {str(e)}"}

    def send_email(
        self,
        inbox="",
        subject="",
        importance="",
        email_body="",
        to_recipients=[],
        cc_recipients=[],
        bcc_recipients=[],
        attachments=[],
    ):
        if inbox == "":
            raise Exception("Inbox is required")

        email_draft = self.draft_email(
            inbox,
            subject,
            importance,
            email_body,
            to_recipients,
            cc_recipients,
            bcc_recipients,
        )

        if attachments != []:
            self._add_attachments_to_email(inbox, email_draft["id"], attachments)

        url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{email_draft['id']}/send"

        headers = {
            "Authorization": f"Bearer {self.token}",
            "Content-type": "application/json",
        }

        try:
            response = requests.post(url, headers=headers)

            if response.status_code == 202:
                print("Email sent successfully ✨")
                return {
                    "status": "success",
                    "message": "Email sent successfully",
                    "recipients": {
                        "to": to_recipients,
                        "cc": cc_recipients,
                        "bcc": bcc_recipients,
                    },
                    "subject": subject,
                    "body": email_body,
                }
            else:
                # Handle error cases gracefully
                error_message = "Failed to send email"
                try:
                    error_data = response.json()
                    if "error" in error_data:
                        error_message = error_data["error"].get(
                            "message", error_message
                        )
                except (json.JSONDecodeError, ValueError, KeyError):
                    error_message = f"HTTP {response.status_code}: {response.reason}"

                return {
                    "status": "error",
                    "message": error_message,
                    "status_code": response.status_code,
                }
        except requests.exceptions.RequestException as e:
            return {"status": "error", "message": f"Request failed: {str(e)}"}

    def _add_attachments_to_email(self, inbox, id, attachments):
        for attachment in attachments:
            if attachment.startswith("http"):
                file_content = self._get_file_from_url(attachment)
            else:
                file_content = self._get_file_from_local_file(attachment)

            attachment = {
                "@odata.type": "#microsoft.graph.fileAttachment",
                "name": file_content["filename"],
                "contentBytes": self._encode_file_into_base64(file_content["content"]),
            }
            url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{id}/attachments"

            headers = {
                "Authorization": f"Bearer {self.token}",
                "Content-type": "application/json",
            }

            response = requests.post(url, headers=headers, json=attachment)
            response.raise_for_status()
        return {
            "status": "success",
            "message": "Attachment added to email successfully",
            "id": response.json()["id"],
        }

    def _get_file_from_local_file(self, file_path):
        with open(file_path, "rb") as file:
            content = file.read()
            filename = os.path.basename(file_path)
            file_extension = os.path.splitext(filename)[1]

            return {
                "content": content,
                "filename": filename,
                "file_extension": file_extension,
                "size": len(content),
            }

    def _get_file_from_url(self, url):
        response = requests.get(url)
        response.raise_for_status()

        # Extract filename from URL
        parsed_url = urlparse(url)
        filename_from_url = os.path.basename(unquote(parsed_url.path))

        # Try to get filename from Content-Disposition header
        filename_from_header = None
        content_disposition = response.headers.get("Content-Disposition", "")
        if "filename=" in content_disposition:
            filename_from_header = content_disposition.split("filename=")[1].strip(
                "\"'"
            )

        # Determine final filename (prefer header over URL)
        filename = filename_from_header or filename_from_url or "downloaded_file"

        # Extract file extension
        file_extension = os.path.splitext(filename)[1] if filename else ""

        return {
            "content": response.content,
            "filename": filename,
            "file_extension": file_extension,
            "size": len(response.content),
        }

    def _encode_file_into_base64(self, file_content):
        return base64.b64encode(file_content).decode("utf-8")



import requests
import base64
from yougotmail._utils._utils import Utils
from yougotmail.storage.storage import Storage
from yougotmail.retrieve.retrieval_utils import RetrievalUtils


class RetrieveAttachments:
    def __init__(
        self,
        client_id,
        client_secret,
        tenant_id,
        mongo_url="",
        mongo_db_name="",
        email_collection="",
        conversation_collection="",
        attachment_collection="",
        aws_access_key_id="",
        aws_secret_access_key="",
        region_name="",
        bucket_name="",
    ):
        self.utils = Utils()
        self.token = self.utils._generate_MS_graph_token(
            client_id, client_secret, tenant_id
        )
        self.db_storage = Storage(
            mongo_url=mongo_url,
            mongo_db_name=mongo_db_name,
            email_collection=email_collection,
            conversation_collection=conversation_collection,
            attachment_collection=attachment_collection,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name,
            bucket_name=bucket_name,
        )
        self.retrieval_utils = RetrievalUtils(client_id, client_secret, tenant_id)

    def get_attachments(
        self,
        inbox,
        range,
        start_date,  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        start_time,
        end_date,  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        end_time,
        subject,
        sender_name,
        sender_address,
        recipients,
        cc,
        bcc,
        folder_path,
        drafts,
        archived,
        deleted,
        sent,
        read,
        storage,
    ):
        try:
            attachments_list = []
            if inbox == []:
                raise Exception("Inbox is required")
            if range != "" and (
                start_date != "" or end_date != "" or start_time != "" or end_time != ""
            ):
                raise Exception(
                    "Start date and end date are not allowed when range is provided"
                )

            for inbox in inbox:
                attachments_from_one_inbox = self._get_attachments_for_one_inbox(
                    inbox,
                    range,
                    start_date,
                    start_time,
                    end_date,
                    end_time,
                    subject,
                    sender_name,
                    sender_address,
                    recipients,
                    cc,
                    bcc,
                    folder_path,
                    drafts,
                    archived,
                    deleted,
                    sent,
                    read,
                )
                if attachments_from_one_inbox is not None:
                    attachments_list.append(attachments_from_one_inbox)
                else:
                    return None

            if storage and len(attachments_list) > 0:
                self.db_storage.store_attachments(attachments_list)
                attachments_list = self.utils._convert_datetimes(
                    self.utils._remove_objectid_from_list(attachments_list)
                )

            return attachments_list

        except Exception as e:
            print(f"Error in get_attachments: {e}")
            return None

    def _get_attachments_for_one_inbox(
        self,
        inbox,
        range,
        start_date,  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        start_time,
        end_date,  # can be date: YYYY-MM-DD or datetime YYYY-MM-DDT00:00:00Z
        end_time,
        subject,
        sender_name,
        sender_address,
        recipients,
        cc,
        bcc,
        folder_path,
        drafts,
        archived,
        deleted,
        sent,
        read,
    ):
        try:
            url_filter = self.retrieval_utils._create_filter_url(
                inbox=inbox,
                range=range,
                start_date=start_date,
                start_time=start_time,
                end_date=end_date,
                end_time=end_time,
                subject=subject,
                sender_name=sender_name,
                sender_address=sender_address,
                recipients=recipients,
                cc=cc,
                bcc=bcc,
                folder_path=folder_path,
                drafts=drafts,
                read=read,
            )

            url = (
                f"https://graph.microsoft.com/v1.0/users/{inbox}/messages?{url_filter}"
            )

            headers = {
                "Authorization": f"Bearer {self.token}",
                "Accept": "application/json",
                "Prefer": 'outlook.body-content-type="text"',
            }

            try:
                output_attachments_list = []
                page_count = 0
                while url and page_count < 100000:
                    page_count += 1
                    response = requests.get(url, headers=headers)
                    response.raise_for_status()
                    data = response.json()
                    emails = data["value"]
                    next_page_link = data.get("@odata.nextLink")

                    for email in emails:
                        attachment_list = self._get_attachments_from_one_email(
                            inbox, email
                        )
                        re_structured_email = self._re_structure_email_for_attachments(
                            email, inbox, attachment_list
                        )
                        filtered_email = self.retrieval_utils._filter_email_outputs(
                            re_structured_email, archived, deleted, sent
                        )
                        if filtered_email is not None:
                            attachments = filtered_email.get("attachments")
                            output_attachments_list.extend(attachments)
                    url = next_page_link

            except requests.exceptions.RequestException as e:
                print(f"Error making request: {e}")
                return None

            if len(output_attachments_list) == 0:
                return None

            return {
                "inbox": inbox,
                "number_of_attachments_found": len(output_attachments_list),
                "attachments": output_attachments_list,
            }
        except Exception as e:
            print(f"Error in _get_attachments_for_one_inbox: {e}")
            return None

    def _get_attachments_from_one_email(self, inbox, email):
        try:
            attachments = []
            if email is not None:
                email_id = email["id"]
                has_attachments = email.get("hasAttachments", False)
                if not has_attachments:
                    return []

            url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{email_id}/attachments"

            headers = {
                "Authorization": f"Bearer {self.token}",
                "Accept": "application/json",
            }

            response = requests.get(url, headers=headers)
            response.raise_for_status()
            data = response.json()
            values = data["value"]

            for value in values:
                attachment_id = value.get("id", "")
                attachment_url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{email_id}/attachments/{attachment_id}/$value"
                attachment_response = requests.get(attachment_url, headers=headers)
                attachment_response.raise_for_status()

                attachment = {
                    "attachment_id": "attachment_" + attachment_id,
                    "file_name": value["name"],
                    "date": value["lastModifiedDateTime"],
                    "contentType": value["contentType"],
                    "contentBytes": base64.b64encode(
                        attachment_response.content
                    ).decode("utf-8"),
                }
                attachments.append(attachment)

            return attachments

        except requests.exceptions.RequestException as e:
            print(f"Error making request: {e}")
            return None

    def _re_structure_email_for_attachments(self, email, inbox, attachment_list):
        unique_body = email.get("uniqueBody", {}).get("content", "")
        if unique_body == "":
            body = email.get("body", {}).get("content", "")
        else:
            body = unique_body
        body = self.retrieval_utils._trim_reply_section(body)

        from_field = email.get("from", {}).get("emailAddress", {})
        sender_address = from_field.get("address", "")
        new_attachment_list = []
        for attachment in attachment_list:
            attachment_object = {
                "inbox": inbox,
                "sender_address": sender_address,
                "file_name": attachment["file_name"],
                "date": attachment["date"],
                "contentType": attachment["contentType"],
                "contentBytes": attachment["contentBytes"],
            }
            new_attachment_list.append(attachment_object)
        return {
            "folder_name": self.retrieval_utils._get_folder_name(
                email["parentFolderId"], inbox
            ),
            "attachments": new_attachment_list,
        }



from yougotmail import YouGotMail
import os
from dotenv import load_dotenv
import json

load_dotenv()

ygm = YouGotMail(
    client_id=os.environ.get("MS_CLIENT_ID"),
    client_secret=os.environ.get("MS_CLIENT_SECRET"),
    tenant_id=os.environ.get("MS_TENANT_ID"),
    open_ai_api_key=os.environ.get("OPENAI_API_KEY"),
)

def test_ai_agent_with_tools():
    try:
        prompt = input("Enter your prompt: ")
        response = ygm.ai_agent_with_tools(
            inbox=os.environ.get("INBOX_1"),
            prompt=prompt
            )
        print(response)
    except Exception as e:
        print(f"Error: {e}")



from setuptools import setup, find_packages

setup(
    name="yougotmail",
    version="0.0.12",
    description="Easily create AI Agents in MS Outlook",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="Witold Kowalczyk",
    author_email="witold@delosintelligence.com",
    url="https://github.com/WitoldKowalczyk/YouGotMail",
    packages=find_packages(),
    include_package_data=True,  # includes files listed in MANIFEST.in (if any)
    install_requires=[
        "requests",
        "python-dateutil",
    ],
    extras_require={"openai": ["openai"], "boto3": ["boto3"], "pymongo": ["pymongo"]},
    classifiers=[
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.10.12",
)
